import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:rapidfire/Db.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:rapidfire/uiux/AlertRF.dart';
import 'package:rapidfire/GlobalRF.dart';

class ListViewRF extends StatefulWidget {
  DataSourceRF dataSource;
  int pageSize;
  final dynamic builder;
  final Function onTap;

  @override
  _ListViewRFState createState() => _ListViewRFState();

  ListViewRF({this.dataSource, this.pageSize = 20, this.builder, this.onTap});
}

class _ListViewRFState extends State<ListViewRF> {
  ScrollController _controller = new ScrollController();

  Future _loadMore(pageNo) async {
    if (!widget.dataSource.isLoading) {
      widget.dataSource.updateSql(pageNo, 'desc');
      await widget.dataSource.read(pageNo, widget.pageSize, this);
    }
  }

  onFilter() {
    widget.dataSource.read(0, widget.pageSize, this);
  }

  @override
  void dispose() {
    //var className = PageRF.callerStack.pop();
    //PageRF.dataSourcesCallerRefreshMap.remove(className);
    super.dispose();
  }

  @override
  void initState() {
    widget.dataSource.onFilter = onFilter;
    widget.dataSource.updateSql(0, 'desc');
    widget.dataSource.read(0, widget.pageSize, this);

    if (PageRF.dataSourcesCallerRefreshMap[PageRF.currentPageClassName] == null) {
      PageRF.dataSourcesCallerRefreshMap[PageRF.currentPageClassName] = [widget.dataSource];
    } else {
      Iterable found = PageRF.dataSourcesCallerRefreshMap[PageRF.currentPageClassName]
          .where((e) => e.model.toString() == widget.dataSource.model.toString());
      if (found == null || found.length == 0) {
        PageRF.dataSourcesCallerRefreshMap[PageRF.currentPageClassName].add(widget.dataSource);
      }
    }

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Stack(
        fit: StackFit.expand,
        children: [
          Column(
            children: <Widget>[
              Expanded(
                flex: 80,
                child: NotificationListener<ScrollNotification>(
                  onNotification: (ScrollNotification scrollInfo) {
                    if (!widget.dataSource.isLoading && scrollInfo.metrics.pixels == scrollInfo.metrics.maxScrollExtent) {
                      _loadMore(widget.dataSource.currentPage++);
                      setState(
                        () => widget.dataSource.isLoading = true,
                      );
                    }
                    return true;
                  },
                  child: ListView.builder(
                    physics: const AlwaysScrollableScrollPhysics(),
                    controller: _controller,
                    itemCount: widget.dataSource.modelList.length,
                    itemBuilder: (context, index) {
                      return buildItems(context, widget.dataSource.modelList[index], index);
                    },
                  ),
                ),
              ),
              Expanded(
                  flex: 25,
                  child: Container(
                    color: Colors.transparent,
                  )),
            ],
          ),
          Positioned(
            top: MediaQuery.of(context).size.height - 190,
            left: 0,
            right: 0,
            child: Visibility(
              visible: widget.dataSource.isLoading,
              child: Container(
                // height: isLoading ? 20.0 : 0,
                height: 50,
                color: Colors.white,
                padding: EdgeInsets.only(top: 12),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Container(
                      height: 20,
                      child: SpinKitWave(color: Colors.black, type: SpinKitWaveType.start, size: 30),
                    ),
                    SizedBox(width: 10),
                    Text('loading, wait for a moment...'),
                  ],
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  buildItems(context, model, index) {
    var pkValue = Db().getPrimaryKeyValue(model);
    return Dismissible(
      key: UniqueKey(),
      confirmDismiss: (direction) async {
        return await AlertRF(
            context: context,
            title: 'Confirm',
            icon: Icons.error,
            msg: 'Do you want to delete this record?',
            confirmText: 'DELETE',
            onConfirm: (value) {
              if (value) {
                if (widget.dataSource.type == DataSourceType.local) {
                  rf.db.delete(model, pkValue);
                  widget.dataSource.modelList.removeAt(index);
                } else if (widget.dataSource.type == DataSourceType.remote) {
                  rf.rdb.delete(model, pkValue);
                  widget.dataSource.modelList.removeAt(index);
                } else if (widget.dataSource.type == DataSourceType.memory) {
                  widget.dataSource.modelList.removeAt(index);
                }
                setState(() {});
                print('deleting...$pkValue');
              }
            }).show();
      },
      background: Container(color: Colors.transparent),
      child: InkWell(
        onTap: () => widget.onTap(model, index),
        child: widget.builder(model, index),
      ),
    );
  }
}
